function W=Decom1D(Method, X, p1, p2)
% Decom1D   Decompose a 1D-signal into expansion coefficients, 
%           or the inverse function if Method is negative.
% Most decompositions methods suppurted by this function are orthogonal.
% Many of these methods also use other functions to do the real work.
% A special non-orthogonal case, frames, is also supported, the frame should
% then be stored in a mat-file with variables as specified in FrameFile.m
%
% Examples:
% W = Decom1D(Method, X);            % decomposition by Method
% X = Decom1D(-Method, W);           % inverse decomposition by Method
% W = Decom1D(Method, X, p1, p2);    % also using parameters p1, p2
% X = Decom1D(-Method, W, p1, p2);   % inverse, using parameters p1, p2
% ---------------------------------------------------------------------------
% arguments:
%  X        the signal, a column vector of real data, size Samplesx1
%  Method   An integer for the decomposition method 
%           Negative sign is used for the inverse function
%           1        - No decomposition 
%           2-128    - 2x2 to 128x128 Discrete Cosine Transform, DCT
%           201-207  - Tree structure IIR filter bank, 2^(Method-200) subbands
%                      Two additional arguments may be given, they are optional
%             p1       a0 used in the IIR filterbank.
%             p2       a1 used in the IIR filterbank.
%                      NOTE: energy of W will be 2^(Method-200) times energy of X
%           210      - 32x16 Lapped Orthogonal Transform, LOT. 
%                      (this is the same as Method 225)
%           211-217  - Dyadic filter bank, Daubechies 7-9 biorthogonal filters
%                      1-7 (Method-210) levels, 2-8 (Method-209) subbands
%           218      - wavelet decomposition (The wavelet toolbox is needed!)
%                      Two additional arguments should be given (default 'db6', 4 levels)
%                      db6 is the Daubechies wavlets, filterlength is 12.
%             p1       the wavelet name, as 'wname' used in 'wfilters' command
%             p2       The number of levels, 1-7.
%           220-229  - 2NxN Lapped Orthogonal Transform, LOT from GetLOT
%                      where we use N=[4,6,8,10,12,16,20,24,32,64].
%                      A third argument may be given, it is optional
%             p1       autocorrelation of signal, as 'rxx' used in 'GetLOT.m'
%           230-239  - 4NxN Extended Lapped Transform, ELT from GetELT
%                      where we use N=[4,6,8,10,12,16,20,24,32,64].
%             p1       free variable p, as 'p' used in 'GetELT.m'
%             p2       autocorrelation of signal, as 'rxx' used in 'GetELT.m'
%           255      - Use the frame stored in FrameFile
%             p1       the name of the mat-file for the frame, FrameFile
%             p2       the sparseness factor to use (override Savg in FrameFile)
%  W        the expansion coefficients, this matrix will be of size KxL
%           where K depends on the decomposition method, and L also 
%           depends on the number of samples, often we will have Samples=K*L
% ---------------------------------------------------------------------------

% This function needs the following functions to be available:
% C_anafb, C_synfb, (which again use C_ana and C_syn), C_anablock, C_synblock
% GetLOT, GetELT, and wfilters in Matlab Wavelet Toolbox.
% For frames: BuildFg, VSblock, GlobalMP  ??

%----------------------------------------------------------------------
% Copyright (c) 1999.  Karl Skretting.  All rights reserved.
% Hogskolen in Stavanger (Stavanger University), Signal Processing Group
% Mail:  karl.skretting@tn.his.no   Homepage:  http://www.ux.his.no/~karlsk/
% 
% HISTORY:  dd.mm.yyyy
% Ver. 1.0  17.07.2000  Karl Skretting, function made in the Frame context
% Ver. 1.1  20.12.2000  KS: some changes 
% Ver. 1.2  22.01.2001  KS: Decomposition using frames added
% Ver. 1.3  13.11.2001  KS: VSblock and VSolap1 possible for frames, Display added
% Ver. 1.4  02.12.2002  KS: moved from ..\Frames to ..\FrameTools
%----------------------------------------------------------------------

Mfile='Decom1D';
Display=1;

% check input and output arguments, and assign values to arguments
if (nargin < 4); p2=[]; end;
if (nargin < 3); p1=[]; end;
if (nargin < 2); 
   error([Mfile,': function must have two input arguments, see help.']); 
end
NofArgs=4;
if length(p2)==0; NofArgs=3; end;
if length(p1)==0; NofArgs=2; end;
if (nargout ~= 1); 
   error([Mfile,': function must have one output arguments, see help.']); 
end
if (Method>0); Decomp=1; else Decomp=0; end;
Method=abs(floor(Method));
if (Method==218) & (NofArgs<4)       % use a default
   p1='db6';p2=4;      
end

% find N and K and t2 (a text describing the method) from the Method
t1=[Mfile,': Method ',int2str(Method),' '];
if sum(Method==[1:128]);       
    N=Method;K=N;
    t2=[t1,int2str(N),'x',int2str(N),' Discrete Cosine Transform, DCT.'];
elseif sum(Method==[201:207]); 
    N=2^(Method-200);K=N;
    t2=[t1,'IIR tree structured filter bank with ',int2str(N),' subbands.'];
elseif (Method==210);          
    N=16;K=N;
    t2=[t1,'16x32 Lapped Orthogonal Transform.'];
elseif sum(Method==[211:217]); 
    N=2^(Method-210);K=N;
    t2=[t1,'Dyadic Daubechies 7-9 filter bank, ',int2str(Method-210),' levels.'];
elseif Method==218; 
    N=2^p2;K=N;
    t2=[t1,p1,'-wavelet  of ',int2str(p2),' levels.'];
elseif sum(Method==[220:229]); 
    if Method==220; N=4; end;
    if Method==221; N=6; end;
    if Method==222; N=8; end;
    if Method==223; N=10; end;
    if Method==224; N=12; end;
    if Method==225; N=16; end;
    if Method==226; N=20; end;
    if Method==227; N=24; end;
    if Method==228; N=32; end;
    if Method==229; N=64; end;
    K=N;
    t2=[t1,int2str(2*N),'x',int2str(N),' Lapped Orthogonal Transform.'];
elseif sum(Method==[230:239]); 
    if Method==230; N=4; end;
    if Method==231; N=6; end;
    if Method==232; N=8; end;
    if Method==233; N=10; end;
    if Method==234; N=12; end;
    if Method==235; N=16; end;
    if Method==236; N=20; end;
    if Method==237; N=24; end;
    if Method==238; N=32; end;
    if Method==239; N=64; end;
    K=N;
    t2=[t1,int2str(4*N),'x',int2str(N),' Extended Lapped Orthogonal Transform.'];
elseif (Method==255)
    FrameFile=p1;N=0;Mdim=0;
    if exist([FrameFile,'.mat'])
        try
            % load the following variables from FrameFile: Class, Type, Mdim, F, SizeF
            % G, Ctab, Dtab, Fbest, Savg, Mdat, PreProc, VecSel, InitialF, History, SNRtot
            load(FrameFile);
            K=SizeF(Mdim+1);
            N=prod(SizeF(1:Mdim));
            P=prod(SizeF((Mdim+2):(2*Mdim+1)));
            S=Savg;
            if (length(p2)); if (p2>0); S=p2; end; end;
            t2=[t1,'using FrameFile ',FrameFile,'.mat. Class=',...
               Class,'  Type=',Type','  Mdim=',int2str(Mdim),'  N=',...
               int2str(N),',  K=',int2str(K),',  P=',int2str(P),'.'];
        catch
            % just continue
        end
    end
    if (N==0) | (Mdim~=1)
        error([Mfile,': Not a frame for 1D-signal in ',FrameFile,'.mat']); 
    end
    if Type=='g'
        Fg=BuildFg(F,G);
    else
        Fg=F;
    end
else
    t2=[Mfile,': An undefined method given.'];
    W=[];
    disp(t2);
    return;
end

if Decomp
    % do decomposition
    disp([t2,' (decomposition).']);
    X=X(:);
    Samples=length(X);
    if (rem(Samples,N) > 0); 
        error([Mfile,': N (as given by Method) is not a factor of length of X.']); 
    end
    L=length(X)/N;
    xx2=X'*X;          
    %
    if (Method==1);   
        W=X;
    elseif sum(Method==[2:128]);       
        W=dct(reshape(X,N,L)); 
    elseif sum(Method==[201:207]); 
        if (NofArgs == 4)
            if (p1*p2)==0
                W=C_anafb(X,N).';    
            else
                W=C_anafb(X,N,p1,p2).';     % use parameters
            end
        else
            W=C_anafb(X,N).';    
        end
    elseif (Method==210);          
        T=GetLOT(16,0.95); 
        T=T';                         % the forward transform  size 16x32   
        W=reshape(X,N,L);             % W is 16xL, L=Samples/16 and N=16 
        W=[W;[W(:,2:L),W(:,1)]];      % W is 32xL  
        W=T*W;                        % W is 16xL
    elseif sum(Method==[211:217]); 
        % The Daubechies 7-9 biorthogonal wavelet is like in table 7.2 page 119
        % in C.S.Burrows et.al "Introduction to Wavelets and Wavelet Transforms"   
        rh=[-0.06453888262876,-0.04068941760920,0.41809227322204,...
            0.78848561640637,0.41809227322204,-0.04068941760920,-0.06453888262876];
        rg=[0.03782845550726,0.02384946501956,-0.11062440441844,-0.37740285561283,...
            0.85269867900889,-0.37740285561283,-0.11062440441844,0.02384946501956,...
            0.03782845550726];
        W=zeros(N,L);                 % N=2^(Method-210)
        x3=C_anablock(X,rh,rg);       % x3 is just used temporary here
        i=Method-211;                 % levels left
        while i
            W(((2^i)+1):(2^(i+1)),:)=reshape(x3(:,2),2^i,L);
            x3=C_anablock(x3(:,1),rh,rg);   
            i=i-1;
        end
        W(1:2,:)=x3';
    elseif (Method==218);             % wavelet
        [h,g,rh,rg] = wfilters(p1);      % building blocks of the filterbank
        W=zeros(N,L);                    % N=2^(Method-210)
        x3=C_anablock(X,h,g);              % x3 is just used temporary here
        i=p2-1;                          % levels left
        while i
            W(((2^i)+1):(2^(i+1)),:)=reshape(x3(:,2),2^i,L);
            x3=C_anablock(x3(:,1),h,g);   
            i=i-1;
        end
        W(1:2,:)=x3';
    elseif sum(Method==[220:229]);    % LOT
        if ((NofArgs>=3) & (p1~=0))
            T=GetLOT(N,p1);
        else
            T=GetLOT(N,0.95);
        end
        T=T';                         % the forward transform  size Nx(2N)   
        W=reshape(X,N,L);             % W is NxL, L=Samples/N  
        W=[W;[W(:,2:L),W(:,1)]];      % W is (2N)xL  
        W=T*W;                        % W is NxL
    elseif sum(Method==[230:239]);    % ELT
        if ((NofArgs==3) & (p1~=0))
            T=GetELT(N,p1,0.95);
        elseif ((NofArgs==4) & ((p1*p2)~=0))
            T=GetELT(N,p1,p2);         % 4NxN
        else
            T=GetELT(N,0.7,0.95);
        end
        T=T';                         % the forward transform  size Nx(4N)   
        W=reshape(X,N,L);             % W is NxL, L=Samples/N  
        W=[W;[W(:,2:L),W(:,1)];[W(:,3:L),W(:,1:2)];[W(:,4:L),W(:,1:3)]];  % W is (4N)xL  
        W=T*W;                        % W is NxL
    elseif Method==255
        % S is Savg stored in the frame or fourth argument (p2) if given
        X=reshape(X,N,L);
        % the variables for the frame is already loaded
        if strcmp(VecSel.Prog1,'VSblock') | strcmp(VecSel.Prog1,'VSolap1')
            % the distribution S(l) is found by VecSel.Prog1, since S is sparseness
            % factor it will initially be evenly distributed, but updated during
            % vector selection. This depends on the arguments B, el, and Bf
            VSalg=VecSel.arg1;
            B=VecSel.arg2;
            el=VecSel.arg3;
            Bf=VecSel.arg4;
            if el<1; el=1; end;
            % we do VSfomp2 first time anyway, (GMP+fomp2)
            if strcmp(VecSel.Prog1,'VSblock')
                W=VSblock(X,Fg,S,'VSfomp2',B,0,Bf);   
                R=X-Fg*W;
            else
                W=VSolap1(X,Fg,S,'VSfomp2',B,0);   
                R=X; for p=1:P; R=R-Fg(:,:,p)*W(:,[(L+2-p):L,1:(L+1-p)]); end;
            end   
            rr2=R(:)'*R(:);
            temp=10*log10(xx2/rr2);
            if Display; 
                disp([Mfile,': after VSfomp2,  SNR=',num2str(temp,4)]);
                for i=1:el
                    if strcmp(VecSel.Prog1,'VSblock')
                        W=VSblock(X,Fg,W,VSalg,B,0,Bf);   
                        R=X-Fg*W;
                    else
                        W=VSolap1(X,Fg,W,VSalg,B,0);   
                        R=X; for p=1:P; R=R-Fg(:,:,p)*W(:,[(L+2-p):L,1:(L+1-p)]); end;
                    end
                    rr2=R(:)'*R(:);
                    temp=10*log10(xx2/rr2);
                    Display; disp([Mfile,': Iteration ',int2str(i),' using ',VSalg,...
                            ',  SNR=',num2str(temp,4)]); 
                end
            else
                if strcmp(VecSel.Prog1,'VSblock')
                    W=VSblock(X,Fg,W,VSalg,B,el-1,Bf);   
                else
                    W=VSolap1(X,Fg,W,VSalg,B,el-1);   
                end
            end
            % S=full(sum(W~=0));
        else
            disp([Mfile,': not ready for vector selection using ',VecSel.Prog1,'.']);
            W=zeros(K,L);
        end
        W=full(reshape(W,K,L));         % W is size KxL
    else
        error([Mfile,': Unknown value of Method, see help.']);
    end
else
    disp([t2,' (reconstruction).']);
    % do inverse decomposition
    W=X(:);
    if (rem(length(W),K) > 0); 
        error([Mfile,': K (as given by Method) is not a factor of length of W.']); 
    end
    L=length(W)/K;
    W=reshape(W,K,L);
    if (Method==1);   
        X=W(:);
    elseif sum(Method==[2:128]);       
        X=idct(W); 
        X=X(:);
    elseif sum(Method==[201:207]); 
        if (NofArgs == 4)
            if (p1*p2)==0
                X=C_synfb(W.');            
            else
                X=C_synfb(W.',p1,p2).';     % use parameters
            end
        else
            X=C_synfb(W.');             % or not
        end
    elseif (Method==210);          
        T=GetLOT(16,0.95); 
        X=T*W;
        X=X(1:N,:)+[X((N+1):(2*N),L),X((N+1):(2*N),1:(L-1))];
        X=X(:);
    elseif sum(Method==[211:217]); 
        % The Daubechies 7-9 biorthogonal wavelet is like in table 7.2 page 119
        % in C.S.Burrows et.al "Introduction to Wavelets and Wavelet Transforms"   
        h=[0.03782845550726,-0.02384946501956,-0.11062440441844,0.37740285561283,...
         0.85269867900889,0.37740285561283,-0.11062440441844,-0.02384946501956,...
         0.03782845550726];
        g=[0.06453888262876,-0.04068941760920,-0.41809227322204,0.78848561640637,...
         -0.41809227322204,-0.04068941760920,0.06453888262876];
        W=W.';
        X=W(:,1:2);
        for i=1:(Method-211)
            X=[C_synblock(X,h,g),reshape(W(:,(2^i+1):(2^(i+1)))',L*2^i,1)];
        end
        X=C_synblock(X,h,g);
        W=W.';
    elseif (Method==218);             % wavelet
        if NofArgs<4       % use a default
            p1='db6';p2=4;
        end
        [h,g,rh,rg] = wfilters(p1);      % building blocks of the filterbank
        W=W.';
        X=W(:,1:2);
        for i=1:(p2-1)
            X=[C_synblock(X,rh,rg),reshape(W(:,(2^i+1):(2^(i+1)))',L*2^i,1)];
        end
        X=C_synblock(X,rh,rg);
        W=W.';
    elseif sum(Method==[220:229]);    % LOT
        if ((NofArgs>=3) & (p1~=0))
            T=GetLOT(N,p1);
        else
            T=GetLOT(N,0.95);
        end
        X=T*W;
        X=X(1:N,:)+[X((N+1):(2*N),L),X((N+1):(2*N),1:(L-1))];
        X=X(:);
    elseif sum(Method==[230:239]);    % ELT
        if ((NofArgs==3) & (p1~=0))
            T=GetELT(N,p1,0.95);
        elseif ((NofArgs==4) & ((p1*p2)~=0))
            T=GetELT(N,p1,p2);         % 4NxN
        else
            T=GetELT(N,0.7,0.95);
        end
        X=T*W;                        % 4NxL
        X=X(1:N,:)+[X((N+1):(2*N),L),X((N+1):(2*N),1:(L-1))]+...
            [X((2*N+1):(3*N),(L-1):L),X((2*N+1):(3*N),1:(L-2))]+...
            [X((3*N+1):(4*N),(L-2):L),X((3*N+1):(4*N),1:(L-3))];     % NxL
        X=X(:);
    elseif Method==255
        if P==1
            X=full(Fg*W);
        else
            temp=W;X=zeros(N,L);
            for p=1:P
                X=X+full(Fg(:,:,p)*temp);   
                temp=[temp(:,L),temp(:,1:(L-1))];
            end
        end
        X=X(:);
    else
        error([Mfile,': Unknown value of Method, see help.']);
    end
    W=X(:);   % output argument name is W
end

return


