function ErrCode=DesignF(FrameFile,TotIt,MaxDays)
% DesignF   Design the frame and store it in FrameFile, based on training data
%           stored in mat-files, DataXnnn.mat, in current directory.
% The function will return after a total of TotIt iterations.
% 
% ErrCode=DesignF(FrameFile,TotIt,MaxDays);
%-----------------------------------------------------------------------------------
% arguments:
%  ErrCode   - 0 is returned if the function execute without error
%              1 an 'unexpected' error occurred, lasterr may explain it
%              2 or larger, another error occurred, 
%  FrameFile - the name of the mat-file used to store the frame
%  TotIt     - total number of iterations to do 
%              If more than TotIt iterations already has been done on this frame
%              then no iterations will be done now, the function just return and
%              do not change the frame in FrameFile.
%  MaxDays   - how many days the design process should be allowed to run
%              if MaxDays=0.25 it is 6 hours, 0.01 is 14 minutes and 24 seconds
%-----------------------------------------------------------------------------------

%----------------------------------------------------------------------
% Copyright (c) 2000.  Karl Skretting.  All rights reserved.
% Hogskolen in Stavanger (Stavanger University), Signal Processing Group
% Mail:  karl.skretting@tn.his.no   Homepage:  http://www.ux.his.no/~karlsk/
% 
% HISTORY:  dd.mm.yyyy
% Ver. 1.0  11.12.2000  KS: function made
% Ver. 1.1  04.10.2001  KS: changed how vector selection is used
%   also the new function NormalizeF is used now
% Ver. 1.2  05.11.2001  KS: Overlapping frame for 2D added, (Mdim==2) & (P>1)
% Ver. 1.3  06.11.2001  KS: Separable frame for 2D added
% Ver. 1.4  03.12.2002  KS: moved from ..\Frames to ..\FrameTools
%----------------------------------------------------------------------

Mfile='DesignF';
ErrCode=0; 
Display=1;
UpdateFforEachSignal=0;

if (nargin<3)
   disp([Mfile,': wrong number of arguments, see help.']);
   ErrCode=2;
   return
end
StopTime=now+MaxDays;

if ~exist([FrameFile,'.mat'])
   disp([Mfile,': the FrameFile ',FrameFile,'.mat does not exist.']);
   ErrCode=3;
   return
end
try
   % load the following variables from FrameFile: Class, Type, Mdim, F, SizeF
   % G, Ctab, Dtab, Fbest, Savg, Mdat, PreProc, VecSel, InitialF, History, SNRtot
   load(FrameFile);
catch
   disp([Mfile,': the FrameFile ',FrameFile,'.mat could not be read.']);
   ErrCode=3;
   return
end
History0=char(History,[Mfile,': Training of ',FrameFile,' started ',datestr(now,0)]);

% The data-files used for training of the frame should be ok and
% stored in the correct mat-files.
% If not, MakeX may be called at this point
% but for now we assume that it is ok.

% check and display the frame type
if Type=='b'
   disp([Mfile,': ',FrameFile,' is a block oriented frame.']);
elseif Type=='o'
   disp([Mfile,': ',FrameFile,' is an overlapping frame.']);
elseif Type=='g'
   disp([Mfile,': ',FrameFile,' is a general frame.']);
elseif Type=='s'
   disp([Mfile,': ',FrameFile,' is a separable frame.']);
elseif Type=='t'
   disp([Mfile,': ',FrameFile,' is a tree structured filter bank (frame), ',...
         Mfile,' is not ready for this yet.']);
   ErrCode=5;
   return
else
   disp([Mfile,': ',FrameFile,' is type ',Type,' which is not allowed.']);
   ErrCode=5;
   return
end
if (length(VecSel.arg4)==0); VecSel.arg4=0; end;
   
% now do the main iterations
% try
   K=SizeF(Mdim+1);
   N=prod(SizeF(1:Mdim));
   P=prod(SizeF((Mdim+2):(2*Mdim+1)));
   if (Mdim==2) & (P>1) 
      Ic=(prod(2*SizeF((Mdim+2):(2*Mdim+1))-1)+1)/2;  % Ic==P when Mdim==1
      %TotWWT=zeros(K,K,Ic);
      TotWWT=zeros(K*P,K*P);    % larger than what is strictly needed
      TotXWT=zeros(N,K*P);
   else
      Ic=(prod(2*SizeF((Mdim+2):(2*Mdim+1))-1)+1)/2;  % Ic==P when Mdim==1
      TotWWT=zeros(K,K,Ic);
      TotXWT=zeros(N,K,P);
   end
   ItNo=size(SNRtot,2);
   Fnormfact=ones(K,1);
   if ItNo>0
      SNRbest=max(SNRtot(Mdat+1,:));
   else
      SNRbest=0;
   end
   if Display
      disp([Mfile,': Training started ',datestr(now,0),', ',int2str(TotIt-ItNo),' it.']);
      t3=num2str(SNRbest,4);t3=[blanks(8-length(t3)),t3];
      disp(['Best SNR is now ',t3]);
   end
   %
   for i=1:(TotIt-ItNo)
      if (now>StopTime); break; end;
      if ~UpdateFforEachSignal
         if (Mdim==2) & (P>1)
            TotWWT=zeros(K*P,K*P);
            TotXWT=zeros(N,K*P);
         else
            TotWWT=zeros(K,K,Ic);
            TotXWT=zeros(N,K,P);
         end
      end
      ex=0;er=0;
      for m=1:Mdat
         if (Mdat>1) | (i==1) | (Type=='s')
            % read the data file
            t1=int2str(m);t1=['0000',t1];t1=t1((length(t1)-2):length(t1));
            DataFile=['DataX',t1]; 
            if ~exist([DataFile,'.mat'])
               disp([Mfile,': the DataFile ',DataFile,'.mat does not exist.']);
               ErrCode=3;
               return
            end
            clear S X W    % make sure that variables are from the DataFile
            load(DataFile);    
            [n,L]=size(X);     % we need the L variable
         end
         disp([Mfile,': use DataFile ',DataFile,'.mat, iteration ',int2str(i),'.']);
         if UpdateFforEachSignal & (i>1)
            TotWWT=TotWWT-WWT;
            TotXWT=TotXWT-XWT;
         end
         % do vector selection
         if Type=='g'
            Fg=BuildFg(F,G);
            Q=size(F,1);
         elseif Type=='s'
            Fv=F{1};
            Fh=F{2};
            Fg=kron(Fh,Fv);
         else
            Fg=F;
         end
         % Fg is now used as the frame, do vector selection
         if strcmp(VecSel.Prog1,'BlockVS')
            disp([Mfile,': vector selection using ',VecSel.Prog1,' is obsolete.']);
            ErrCode=4;
            return
            % koden er i slutten av fila
         elseif strcmp(VecSel.Prog1,'VSblock')
            % first find S (used as third argument in VSblock)
            if (i==1); S=Savg; end;
            % when i>1, S is stored in DataFile (except for 'VSab2' which store W)
            if strcmp(VecSel.arg1,'VSab2') & (i>1)
               W=(Fnormfact(:)*ones(1,L)).*W;   % W was stored instead of S
               S=W;              % and also used as third argument in VSblock
            end
            %if strcmp(VecSel.arg1,'VSab2') & (i==1)
            %   [W,S]=GlobalMP(X,Fg,Savg);    % first time GlobalMP
            %else
            %   disp([Mfile,': do vector selection using ',VecSel.Prog1,'.']);
            %                      VSalg           B          el          Bf
            W=VSblock(X,Fg,S,VecSel.arg1,VecSel.arg2,VecSel.arg3,VecSel.arg4);   
            %end
            S=full(sum(W~=0));
         elseif strcmp(VecSel.Prog1,'VSolap1')
            % first find third argument in VSolap1
            if (i==1)
               W=Savg; 
            else
               W=(Fnormfact*ones(1,L)).*W;   % W was stored instead of S
            end
            %
            W=VSolap1(X,Fg,W,VecSel.arg1,VecSel.arg2);   
            % S=full(sum(W~=0));  % is not used ??
         elseif strcmp(VecSel.Prog1,'VSolap2')
            % first find third argument in VSolap2
            if (i==1)
               W=Savg; 
            else
               W=(Fnormfact*ones(1,L)).*W;   % W was stored instead of S
            end
            %
            W=VSolap2(X,Fg,W,VecSel.arg1,VecSel.arg2);   
            % S=full(sum(W~=0));  % is not used ??
         elseif strcmp(VecSel.Prog1,'GlobalMP')
            disp([Mfile,': vector selection using ',VecSel.Prog1,' is obsolete.']);
            ErrCode=4;
            return
            % [W,S]=GlobalMP(X,Fg,Savg);  
         else
            disp([Mfile,': not ready for vector selection using ',VecSel.Prog1,'.']);
            ErrCode=4;
            return
         end
         % calculate WWT and XWT
         if Type=='s'
            % do not use WWT or XWT, the weights W are stored directly
            WWT=[];XWT=[];
         elseif (P==1)               % for all values of Mdim !!
            WWT=full(W*W');   
            XWT=full(X*W');   
         else 
            if Mdim==1      
               temp=W;
               for p=1:P
                  WWT(:,:,p)=full(W*temp');   
                  XWT(:,:,p)=full(X*temp');   
                  temp=[temp(:,L),temp(:,1:(L-1))];
               end
            elseif (Mdim==2) & (P==4) & (L==4096)
               % WWT (PKxPK) should be Wstar (PKxL) * Wstar' (LxPK)
               % XWT (NxPK) should be X (NxL) * Wstar' (LxPK)    
               % W (KxL) 
               L1=sqrt(L);L2=sqrt(L);  % ==64
               Wstar=sparse(P*K,L);
               for p1=0:1
                  for p2=0:1
                     p=p1+p2*2;
                     Wt=reshape(full(W),K,L1,L2);    % KxL1xL2
                     if p1
                        Wt1=Wt;            
                        Wt(:,1,:)=Wt1(:,L1,:);
                        Wt(:,2:L1,:)=Wt1(:,1:(L1-1),:);
                     end
                     if p2
                        Wt1=Wt;            
                        Wt(:,:,1)=Wt1(:,:,L2);
                        Wt(:,:,2:L2)=Wt1(:,:,1:(L2-1));
                     end
                     Wt=reshape(Wt,K,L);
                     Wstar((1:K)+p*K,:)=sparse(Wt);
                  end
               end
               clear Wt Wt1
               WWT=full(Wstar*Wstar');
               XWT=full(X*Wstar');
               clear Wstar
            else
               disp([Mfile,': overlapping frame and 2D or 3D signal, not ready yet.']);
               ErrCode=4;
               return
            end
         end
         %
         if strcmp(VecSel.arg1,'VSab2') | strcmp(VecSel.Prog1,'VSolap1') ...
               | strcmp(VecSel.Prog1,'VSolap2') | Type=='s'
            %disp([Mfile,': save ',DataFile,', here W is saved.']);
            save(DataFile,'Name','X_DC','X_LP','X','SizeX','ss2','W','WWT','XWT');
         else
            %disp([Mfile,': save ',DataFile,', here S is saved.']);
            save(DataFile,'Name','X_DC','X_LP','X','SizeX','ss2','S','WWT','XWT');
         end
         %
         ex=ex+ss2;    
         if P==1
            R=X-full(Fg*W);
         elseif Mdim==1
            temp=W;R=X;
            for p=1:P
               R=R-full(Fg(:,:,p)*temp);   
               temp=[temp(:,L),temp(:,1:(L-1))];
            end
         elseif (Mdim==2) & (P==4) & (L==4096)
            % this is the special 2D overlapping case handled
            R=X;
            L1=sqrt(L);L2=sqrt(L);  % ==64
            for p1=0:1
               for p2=0:1
                  p=p1+p2*2;
                  Wt=reshape(full(W),K,L1,L2);  % 3D is not fully handled as sparse
                  if p1
                     Wt1=Wt; 
                     Wt(:,1,:)=Wt1(:,L1,:);
                     Wt(:,2:L1,:)=Wt1(:,1:(L1-1),:);
                  end
                  if p2
                     Wt1=Wt; 
                     Wt(:,:,1)=Wt1(:,:,L2);
                     Wt(:,:,2:L2)=Wt1(:,:,1:(L2-1));
                  end
                  Wt=reshape(Wt,K,L);
                  R=R-Fg(:,:,p+1)*Wt;
               end
            end
            clear Wt Wt1
         end
         EnergyR=R(:)'*R(:);
         er=er+EnergyR;
         %
         if Type~='s'
            TotWWT=TotWWT+WWT;
            TotXWT=TotXWT+XWT;
         end
         SNRtot(m,ItNo+i)=10*log10(ss2/EnergyR);
         if m==Mdat
            SNRtot(Mdat+1,ItNo+i)=10*log10(ex/er);
         end
         %
         if Display
            t1=int2str(i);t1=[blanks(4-length(t1)),t1];
            t2=int2str(m);t2=[blanks(4-length(t2)),t2];
            t3=num2str(SNRtot(m,ItNo+i),4);t3=[blanks(8-length(t3)),t3];
            disp(['Find W, iteration ',t1,'  signal ',t2,' (',Name,') gives SNR =',t3]);
         end
         %   
         if (UpdateFforEachSignal & i>1) | (m==Mdat)
            if Type=='g'
               % update of F must be done by using C and D matrices
               C=zeros(Q,Q);
               for i1=1:size(Ctab,1)
                  ci=Ctab(i1,1);ai=Ctab(i1,2);
                  C(ci)=C(ci)+sign(ai)*TotWWT(abs(ai));
               end
               C=C+C'-diag(diag(C));   % IMPORTANT!               
               D=zeros(Q,1);
               for i1=1:size(Dtab,1)
                  di=Dtab(i1,1);bi=Dtab(i1,2);
                  D(di)=D(di)+sign(bi)*TotXWT(abs(bi));
               end
               F=CalculateFg(C,D,G,10000);            % almost F=inv(C)*D
               [F,Fnormfact]=NormalizeF(F,G);
               % if (i==1); save('temp','C'); end;
            elseif Type=='s'
               % This is the special separable frame
               [N1,K1]=size(Fv);
               [N2,K2]=size(Fh);
               K=K1*K2;N=N1*N2;        % Fg is size NxK
               for i1=1:20
                  Fvhold=[Fv(:);Fh(:)]; 
                  % first update Fv
                  TotXWT=zeros(N1,K1);
                  TotWWT=zeros(K1,K1);
                  for m=1:Mdat
                     t1=int2str(m);t1=['0000',t1];t1=t1((length(t1)-2):length(t1));
                     DataFile=['DataX',t1]; 
                     load(DataFile); % saved in line 260, where X is NxL and W is KxL
                     [n,L]=size(X);  % we need the L variable
                     X=reshape(X,N1,N2*L);
                     W=reshape(full(W),K1,K2,L);
                     Wv=zeros(K1,N2,L);
                     for l=1:L; Wv(:,:,l)=W(:,:,l)*(Fh'); end;
                     Wv=reshape(Wv,K1,N2*L);
                     if 0  % test
                        R=X-Fv*Wv;
                        EnergyR=R(:)'*R(:);
                        disp(['Fv before iteration ',int2str(i1),' image ',Name,...
                              ' has SNR ',num2str(10*log10(ss2/EnergyR))]);
                     end
                     TotXWT=TotXWT+X*Wv';
                     TotWWT=TotWWT+Wv*Wv';
                  end
                  Fv=CalculateF(TotWWT,TotXWT,Fv,400);   % almost Fv=TotXWT*inv(TotWWT)
                  % then update Fh
                  TotXWT=zeros(N2,K2);
                  TotWWT=zeros(K2,K2);
                  for m=1:Mdat
                     t1=int2str(m);t1=['0000',t1];t1=t1((length(t1)-2):length(t1));
                     DataFile=['DataX',t1]; 
                     load(DataFile); % saved in line 260, where X is NxL and W is KxL
                     [n,L]=size(X);  % we need the L variable
                     X=reshape(X,N1,N2,L);
                     W=reshape(full(W),K1,K2,L);
                     Wh=zeros(K2,N1,L);   % actually Wh'
                     for l=1:L; Wh(:,:,l)=(Fv*W(:,:,l))'; end;
                     if 0  % test X (N1xN2xL) Wh(') (K2xN1xL) Fh (N2xK2)
                        R=X; for l=1:L; R(:,:,l)=X(:,:,l)-(Fh*Wh(:,:,l))'; end;
                        EnergyR=R(:)'*R(:);
                        disp(['Fh before iteration ',int2str(i1),' image ',Name,...
                              ' has SNR ',num2str(10*log10(ss2/EnergyR))]);
                     end
                     Wh=reshape(Wh,K2,N1*L);  
                     X=permute(X,[2,1,3]);    % change dim 1 and 2, X(:,:,l)=X(:,:,l)'
                     X=reshape(X,N2,N1*L);
                     TotXWT=TotXWT+X*Wh';
                     TotWWT=TotWWT+Wh*Wh';
                  end
                  Fh=CalculateF(TotWWT,TotXWT,Fh,400);   % almost Fh=TotXWT*inv(TotWWT)
                  change=norm(Fvhold-[Fv(:);Fh(:)]);
                  Fvhold=[Fv(:);Fh(:)];    
                  if Display
                     disp(['done Fv and Fh ',int2str(i1),...
                           ' times, change is ',num2str(change)]);
                  end
                  if (change < 1e-5); break; end;
                  if (change > 1000); 
                     disp(['done Fv and Fh ',int2str(i1),' times, change is ',...
                           num2str(change),' no convergence(?) !!']);
                     break 
                  end;
               end
               [Fh,Fhn]=NormalizeF(Fh);
               [Fv,Fvn]=NormalizeF(Fv);
               Fnormfact=kron(Fhn,Fvn);
               F{1}=Fv;
               F{2}=Fh;
               % the end for Type=='s'
            else         % Type=='b'  or Type=='o'  (if P>1)
               if (Mdim==2) & (P==4) & (L==4096)
                  % this is the special 2D overlapping case handled
                  Fstar=CalculateF(TotWWT,TotXWT,reshape(F,N,K*P),400);
                  F=reshape(Fstar,N,K,P);
                  clear Fstar
                  [F,Fnormfact]=NormalizeF(F);
               else
                  % the normal (block-oriented or 1D overlap) case
                  F=CalculateF(TotWWT,TotXWT,F,400);   % almost F=TotXWT*inv(TotWWT)
                  [F,Fnormfact]=NormalizeF(F);
               end
            end
         end
         if (now>StopTime); break; end;
      end   % for m
      %
      if (m==Mdat)    % this is the normal situation      
         if SNRtot(Mdat+1,ItNo+i)>SNRbest
            SNRbest=SNRtot(Mdat+1,ItNo+i);
            Fbest=F;
         end
         %
         History=char(History0,[Mfile,': ',datestr(now,0),' done ',...
               int2str(i),' iterations.']);
         save(FrameFile,'Class','Type','Mdim','F','SizeF','G','Ctab','Dtab',...
            'Fbest','Savg','Mdat','PreProc','VecSel','InitialF','History','SNRtot');
         %
         if Display
            t1=int2str(i);t1=[blanks(4-length(t1)),t1];
            t3=num2str(SNRtot(Mdat+1,ItNo+i),4);t3=[blanks(8-length(t3)),t3];
            disp([FrameFile,': Iteration ',t1,'  for all signals gives SNR =',t3]);
         end
      end
      %       
   end   % for i
   %catch
  % ErrCode=1;
  %end

return;


%-----------------------------------------------------------------------------------
% F=CalculateF(A,B,F,Ratio)    essentially this is: F=B*inv(A) 
% (problem if A is non-invertible is handled), usually A is ivertible  
% arguments:
%   Ratio  the ratio of largest diagonal element on to small elements in A
%          possible values could be: 400, 1000, 160*K*Savg (Savg=1/16 f.ex) 
function F=CalculateF(A,B,F,Ratio)
[N,K,P]=size(F);     
if P>1       % overlappping frame (and 1D signal)
   % input A is now different submatrices of the Cstar matrix
   % also B is submatrices of the D matrix (XW')
   % the current map from Cp to A is valid for 1D overlap only!!
   Cp=A;    % size should be KxKxP
   A=zeros(K*P,K*P);
   for p1=0:(P-1)
      for p2=0:(P-1)
         if p1>p2
            A((1:K)+p1*K,(1:K)+p2*K)=Cp(:,:,p1-p2+1)';
         else
            A((1:K)+p1*K,(1:K)+p2*K)=Cp(:,:,p2-p1+1);
         end
      end
   end
   B=reshape(B,N,K*P);   
   F=reshape(F,N,K*P);   
end
Ad=diag(A);     % all these elements are non-negative
[Adma,Ima]=max(Ad);
SmallAd=find((Ratio*Ad)<Adma);
if (length(SmallAd) | (rcond(A)<1e-6))  
   if 1
      disp(['Special treatment for ',num2str(length(SmallAd)),' vectors.']);
   end
   Ftemp=F;
   % there may be different ways to deal with singular A matrix
   if length(SmallAd)==0
      % we add values to the diagonal to avoid sigular matrix A
      disp(['Add a little bit to the diagonal of A.']);
      A=A+eye(length(Ad))*Adma/Ratio;
      F=B/A;     % and update F the usual way
   else
      % we use only the larger ones
      Ad=diag(A); 
      LargeAd=find((Ratio*Ad)>=Adma);
      disp(['Update the large ones in a normal way.']);
      F(:,LargeAd)=B(:,LargeAd)/A(LargeAd,LargeAd);     % and update F
      % the rarly used ones are changed in another way 
      [temp,Ima]=sort(-Ad);    % sort Ad with largest first
      if 0
         disp(['Update the small ones by adding factor of other vectors.']);
         for i1=1:length(SmallAd); 
            if SmallAd(i1)<=K
               f=(Adma+Ad(SmallAd(i1))*Ratio*0.98)/(2*Adma);  %  0.5 <= f <= 0.99
               t1=rem(Ima(i1)-1,K)+1;
               for p=0:(P-1)
                  F(:,SmallAd(i1)+p*K)=f*Ftemp(:,SmallAd(i1)+p*K)+(1-f)*Ftemp(:,t1); 
               end
            end
         end 
      else
         disp(['Update the small ones by adding noise to the largest ones.']);
         F(:,SmallAd)=F(:,Ima(1:length(SmallAd)))+randn(size(F,1),length(SmallAd))*0.01;
      end
   end
else
   % this is the standard way to update F
   F=B/A;
end
% now F is NxKP, we should resize F
if P>1; F=reshape(F,N,K,P); end;
%
return

%-----------------------------------------------------------------------------------
% F=CalculateFg(C,D,G,Ratio)    essentially this is: F=inv(C)*D
% (problem if C is non-invertible is handled), usually C is ivertible  
% arguments:
%   Ratio  the ratio of largest diagonal element on to small elements in C
%          possible values could be: 400, 1000, 160*K*Savg (Savg=1/16 f.ex) 
%   C      a QxQ matrix
%   D      a Qx1 vector
%   F      and the result is a Qx1 vector
function F=CalculateFg(C,D,G,Ratio)
d=diag(C);             % all these elements are non-negative
F=zeros(size(D));
[d_max,temp]=max(d);
Small_d=find((Ratio*d)<d_max);
if length(Small_d)
   disp(['Special treatment for ',num2str(length(Small_d)),' variables:']);
   % Small_d
   Large_d=find((Ratio*d)>=d_max);
   F(Large_d)=C(Large_d,Large_d)\D(Large_d);     % and update F
   % give som random values to the other values
   temp=mean(abs(F(Large_d)));
   if 1   % try 'low-pass'
      temp2=filter(fir1(9,0.1+0.4*rand),1,randn(length(Small_d)+10,1));
      F(Small_d)=temp*temp2(11:length(temp2));
   else   % just random
      F(Small_d)=0.5*temp*randn(length(Small_d),1);
   end   
   % save('temp','d');
else
   F=C\D;
end
%
return


% OLD code segments, not active now

% kode for VS, if strcmp(VecSel.Prog1,'BlockVS')
if VecSel.arg4
   if ~mod((i-1), VecSel.arg4)  
      [W,S]=GlobalMP(X,Fg,Savg);  
   end
else
   if (i==1)
      % perhaps this should be done for all i, by using the returned
      % S over and over again, it may become "corrupted"
      S=zeros(1,L);
      t1=0;
      for l=1:L
         t1=t1+N*Savg;
         S(l)=floor(t1);
         t1=t1-S(l);
      end
   end
end
if strcmp(VecSel.arg1,'VSab2')
   if (i==1)
      W=S;
   else
      W=(Fnormfact*ones(1,L)).*W;
   end
   [W,S]=BlockVS(X,Fg,W,VecSel.arg1,VecSel.arg2,VecSel.arg3);   
else
   % disp([Mfile,': start BlockVS, sum(S)=',int2str(sum(S)),'.']);
   [W,S]=BlockVS(X,Fg,S,VecSel.arg1,VecSel.arg2,VecSel.arg3);   
   % disp([Mfile,': ended BlockVS, sum(S)=',int2str(sum(S)),'.']);
end
