% FrameFile   Explain the contents of a mat-file that stores a frame. 
%             A frame, as used by the files in this catalog, is a quite
% general concept, and can have a lot of variants. A simple frame does not
% need all these variables, in that case the variables are stored as empty
% arrays, or as variables with value zero.
% For information on a given mat-file, see FrameInfo
%
% The specification or structure for a frame is stored in a mat-file, FrameFile,
% this file should contain the following variables
%  Class     - a character array containing the name of the signal class
%              the frame is adapted for, ex: 'bwImages','ecg','AR(1)'
%  Type      - a single character indicating the type of the frame, [1]
%              The first tree are all frames where the synthesis equation
%              is linear also with regard to the frame coefficients
%               b: block-oriented frame
%               o: overlapping frame
%               g: general frame
%              The last two are special in the way that the (total) frame is
%              not linear with regard to the frame coefficients.
%               s: separable frame 
%               t: tree structured filter bank, i.e. a tree of general filter banks
%                  or general frame, this is not implemented yet
%  Mdim      - number of dimensions in the signal, usually 1 or 2
%  F         - this one is used to store the free variables for the last frame.
%              If Type='b' the size is NxK no matter the dimension. 
%              If Type='o' the size of F is NxKxP no matter the dimension.
%              If Type='g' the size is Qx1 no matter the dimension. 
%              If Type='s' F is a cell array of Mdim matrices
%              If Type='t' F is a cell array of vectors, size (Qi)x1
%  SizeF     - This should be the 'original' size of F, ex 2D signal: N1xN2xKxP1xP2
%  G         - the "mapping" from F to Fg, only used if Type='g', size is NxKxP
%  Ctab      - a map from WWT (size KxKxIc) to C (size QxQ)          
%  Dtab      - a map from XWT (size NxKxP) to D (size Qx1)          
%  Fbest     - as F, but the best values during training are stored here
%  Savg      - the average sparseness factor for each signal, 0<Savg<1
%  Mdat      - Number of signals, (images), used during training of the frame
%  PreProc   - a structure array telling how the signal is preprocessed
%              Prog1   - name of program used to subtract LP part of signal
%                        if DC no other program is needed, mean is subtracted
%                        ImDC is another alternative. 
%              Prog2   - name of preprocess program, possible programs are:
%                        Decom2D, Decom1D, Ttimes, or another m-file used to
%                        generate the set of training vectors.
%              Method  - Method used by Prog2, integer (not necessarily used)
%              argx    - x=1,2,3 arguments passed to Prog2
%  VecSel    - a structure array telling how vector selection is done during training
%              The structure has 6 fields:
%              Prog1   - name of vector selection program, possible programs are:
%                        BlockVS, GlobalMP, VSblock, VSolap1
%              argx    - x=1,2,3,4,5  arguments passed to Prog1 after X,F,S
%                        see help for each function for these.
%  InitialF  - a structure array telling how the initial values of F was set
%              Prog1   - name of program, Possible programs are: 
%                        SetFxx
%              argx    - x=1,2,3,4,5  arguments passed to Prog1
%  History   - a character array telling when and how the frame was made and trained
%  SNRtot    - an array of size 1xItNo or (Mdat+1)xItNo which stores the SNR or PSNR
%              after each time Vector Selection is done. ItNo is
%              number of iterations done 
%                
% The commands that define an empty frame could be like this:
% FrameFile='test';
% PreProc=struct('Prog1','','Prog2',Mfile,'Method',[],'arg1',0,'arg2',0,'arg3',[]);
% VecSel=struct('Prog1','VSblock','arg1','VSfomp2','arg2',1,'arg3',[],'arg4',[],'arg5',[]);
% InitialF=struct('Prog1','','arg1',[],'arg2',[],'arg3',[],'arg4',[],'arg5',[]);
% History='';SNRtot=[];Type='b';Mdat=1;F=[];G=[];Dtab=[];Ctab=[];Savg=0.1;
% SizeF=[5,5,50,1,1];Mdim=2;Class='test';Fbest=[];
% save(FrameFile,'Class','Type','Mdim','F','SizeF','G','Ctab','Dtab',...
%      'Fbest','Savg','Mdat','PreProc','VecSel','InitialF','History','SNRtot');

%----------------------------------------------------------------------
% Copyright (c) 2002.  Karl Skretting.  All rights reserved.
% Hogskolen in Stavanger (Stavanger University), Signal Processing Group
% Mail:  karl.skretting@tn.his.no   Homepage:  http://www.ux.his.no/~karlsk/
% 
% HISTORY:  dd.mm.yyyy
% Ver. 1.0  25.12.2002  KS: text moved from ..\Frames\readme.m to ..\FrameTools
%----------------------------------------------------------------------

type FrameFile
return;

