function  varargout=PlotF2D(F,TransMet,BlkDist,FontSize)
% PlotF2D    Plot the image blocks in frame F, F is size NxK
%            and N is 4, 16, 64 or 256, TransMet must correspond to N
% BlkSize is then given by N (or TransMet), blocks are square.
% Image is made by takeing a sparse W with some few seperated ones, and 
% do the reconstruction process on this W. Each block of this image is
% the the result of only one of the vectors in F.
% Before display these blocks may be seperated by "BlkDist" pixels,
% and blocks corresponding to a certain vector in F may be numbered.
%
% PlotF2D(F,TransMet,BlkDist,FontSize);
% Ir=PlotF2D(F,TransMet,BlkDist,FontSize);
% PlotF2D(eye(64),7,4,8);   % plot the 8x8 DCT basis images
%----------------------------------------------------------------------
% arguments:
%  F         The representation vectors or dictionary, size NxK.
%            if F is scalar and have values 1,2 or 3 we use the initial frames 
%            made by SetF01, SetF02 and SetF03 respectivly.
%  TransMet  An integer for the decomposition method that will be done.
%            Note that this also give the blocksize.
%            This is almost like used in Decom2D
%                1- 4 : NxN identity,    N=[2,4,8,16]
%                5- 8 : NxN DCT,         N=[2,4,8,16]
%                9-12 : 2N*N LOT,        N=[2,4,8,16]
%               13-16 : 4N*N ELT,        N=[2,4,8,16]
%  BlkDist   Distance between each block (in pixels)
%  FontSize  >0 : Display numbers below each block using FontSize
%  Ir        The image that is plotted, (no distance between blocks)
%----------------------------------------------------------------------

%----------------------------------------------------------------------
% Copyright (c) 2000.  Karl Skretting.  All rights reserved.
% Hogskolen in Stavanger (Stavanger University), Signal Processing Group
% Mail:  karl.skretting@tn.his.no   Homepage:  http://www.ux.his.no/~karlsk/
% 
% HISTORY:
% Ver. 1.0  18.08.2000  KS: made function
% Ver. 1.1  29.08.2000  KS: changed input (and added output) arguments
% Ver. 1.2  28.10.2001  KS: changed name to PlotF2D, and some minor changes
% Ver. 1.3  27.11.2002  KS: moved from ..\Frames2D to ..\FrameTools
%----------------------------------------------------------------------

% Reorder og Decom2D m ogs vre tilgjengelige!

Mfile='PlotF2D';

% check input arguments, and assign values to arguments
if (nargin < 2); 
   error([Mfile,': function must have 2 input arguments, see help.']); 
end
if (nargin < 3); BlkDist=0; end;
if (nargin < 4); FontSize=0; end;

TransMet=floor(abs(TransMet));
Mb=2^(mod(TransMet-1,4)+1);
Mb=floor(abs(Mb(1)));
[N,K]=size(F);
if (N*K)==1
   if F==1
      [F,History,PSNRtot]=SetF2d01('test',TransMet,1);
   elseif F==2
      [F,History,PSNRtot]=SetF2d02('test',TransMet,1);
   elseif F==3
      [F,History,PSNRtot]=SetF2d03('test',TransMet,1);
   else
      error([Mfile,': F is scalar but not a legal value, see help.']); 
   end
   [N,K]=size(F);
end
Nb=Mb;                                  % only square blocks 
y=ceil(sqrt(K));
x=ceil(K/y);
if (x*y)>K; F=[F,ones(N,x*y-K)]; end;   % add some empty blocks
[N,K2]=size(F);
Mi=Mb*x;
Ni=Nb*y;
if (N ~= (Mb*Nb)); 
   error([Mfile,': size of F do not correspond to decomposition method, see help.']); 
end

% find expansion factor large enough to avoid overlapping
if sum(TransMet==[1:8]); Efactor=1; end;
if sum(TransMet==[9:12]); Efactor=2; end;
if sum(TransMet==[13:16]); Efactor=4; end;
L=K2*Efactor*Efactor;
% make the sparse W, K ones spread around
W=sparse(K2,L);
for i=0:(x-1)
   for j=0:(y-1)
      k=i*y+j+1;
      if k<=K
         l=i*(Efactor*Efactor*y)+j*Efactor+1;
         W(k,l)=1;
      end
   end
end
%
BlkSize=[Mb,Nb];     % the "small" block
Mb=Mb*Efactor;
Nb=Nb*Efactor;
Mi=Mb*x;
Ni=Nb*y;
ImSize=[Mi,Ni];      % the "large/expanded" image
Xr=full(F*W);
Yr=Reorder(Xr,ImSize,BlkSize,-1);      
Ir=Decom2D(-TransMet,Yr);
% put in distance to get the expanded image (includes space between blocks)
IrMax=max(Ir(:));
IrMin=min(Ir(:));
Ie=IrMax*ones([Mi+BlkDist*(x+1),Ni+BlkDist*(y+1)]);
for i=0:(x-1)
   for j=0:(y-1)
      k=i*y+j+1;
      if k<=K
         Ie(BlkDist+i*(Mb+BlkDist)+(1:Mb),BlkDist+j*(Nb+BlkDist)+(1:Nb))=Ir(i*Mb+(1:Mb),j*Nb+(1:Nb));
      end
   end
end
%
clf
IrMax=max(abs([IrMax,IrMin]));
IrMax=0.3;
imagesc(Ie,[-IrMax,IrMax]);
colormap(gray(256));
% colorbar;
axis off;
axis equal;
hold on;
if FontSize
   for i=0:(x-1)
      for j=0:(y-1)
         k=i*y+j+1;
         if k<=K
            H=text(j*(Nb+BlkDist)+BlkDist+1,(i+1)*(Mb+BlkDist)+0.5,int2str(k));
            set(H,'FontSize',FontSize);
            set(H,'VerticalAlignment','top');
            set(H,'HorizontalAlignment','left');
            if Efactor > 1
               % now plot boxes to indicate how blocks overlap
               i1=i*(Mb+BlkDist)+BlkDist+0.5+Mb*(Efactor-1)/(2*Efactor);
               i2=i*(Mb+BlkDist)+BlkDist+0.5+Mb*(Efactor+1)/(2*Efactor);
               j1=j*(Mb+BlkDist)+BlkDist+0.5+Mb*(Efactor-1)/(2*Efactor);
               j2=j*(Mb+BlkDist)+BlkDist+0.5+Mb*(Efactor+1)/(2*Efactor);
               plot([j1,j1,j2,j2,j1],[i1,i2,i2,i1,i1],'-k');
            end
         end
      end
   end
   H=text(size(Ie,2)-BlkDist,size(Ie,1)+BlkDist,[Mfile,': plot made ',datestr(now,0)]);
   set(H,'FontSize',FontSize);
   set(H,'VerticalAlignment','top');
   set(H,'HorizontalAlignment','right');
end
hold off;
if (nargout>=1); varargout(1)={Ir}; end;

return
