function w=VSfomp2(x,S)
% VSfomp2   Fast Orthogonal Matching Pursuit 2, Vector Selection algorithm 
%           as implemented by A. Barkhald 1999, (Matlab optimized version of VSfomp)
% Compared to VSfomp the differences are
%   VSfomp2 always select two or more vectors
%   VSfomp2 does not return when error is small, r'*r < Arr
%   VSfomp2 depends on that the vectors in F are normalized!
%  
% w=VSfomp2(x,S);
%-----------------------------------------------------------------------------------
% arguments:
%   w    - the weights where at most S of the elements are non-zero, size Kx1
%   x    - the vector to approximate, size Nx1
%   S    - number of vectors to select or non-zero weights 
%-----------------------------------------------------------------------------------
% global variables:
%   F    - the normalized dictionary (or F matrix), size NxK
%          (this matrix is called U in some papers describing OMP/ORMP/FOMP)
%   FF   - the inner products F'*F, size KxK
%-----------------------------------------------------------------------------------

%----------------------------------------------------------------------
% Copyright (c) 1999.  Andre' Barkhald.  All rights reserved.
% Hogskolen in Stavanger (Stavanger University), Signal Processing 
% Mail:  barkhald@online.no   
% 
% HISTORY:  dd.mm.yyyy
% Ver. 1.0  15.02.1999  AB: function made as part of diploma 99,
% Ver. 1.1  04.11.1999  KS: function adjusted to a "stand alone" version
% Ver. 2.0  23.03.2000  KS: changed the arguments used 
% Ver. 2.1  07.04.2000  KS: added ConditionLimit, and maybe call VSmp2 
% Ver. 2.2  11.12.2000  KS: use some global variables 
% Ver. 2.3  25.11.2002  KS: moved from ..\Frames to ..\FrameTools
%-----------------------------------------------------------------------------------

global F FF

Mfile='VSfomp2';
%ConditionLimit=0;        % Do not test for condition number
ConditionLimit=1e5;    % this test is slow, and rarely neccessary
[N,K]=size(F);
if nargin<2
   error([Mfile,': wrong number of arguments, see help.']);
end

us=ones(1,K);   % Since F (frame) is normalized
r=zeros(S,K);

c=x'*F;        % the inner products
% Select the first vector
[value,Jk] = max(abs(c));
Jk=Jk(1);        % only use first element
best  = Jk;      % Pointer to dictionary vector with the best fit
bestC = c(Jk);   % value of signal/dictionary vector-fit
% then the next (second)
k=1;             
r(k,:) = FF(Jk,:);   % Update r
r(k,Jk) = 1;         % just reduces arithmetic op.
c = c - c(Jk)*r(k,:);
u = sqrt(us - r(k,:).^2);
zeroindexes=find(u==0);        % added by KS
u(zeroindexes)=0.001;          % added by KS 30.4.99  
u(best)=1;
c = c./u;
[value,Jk] = max(abs(c));
Jk=Jk(1);        % only use first element
best=[best;Jk];
bestC=[bestC;c(Jk)];
% and now the rest (third to S)
for k=2:(S-1)                
   r(k,:) = (FF(Jk,:) - r(1:k-1,Jk).'*r(1:k-1,:))/u(Jk);
   r(k,Jk) = u(Jk);
   u1=u;
   u = sqrt(u1.^2 - r(k,:).^2);   % 7.6.99  Note that in certain cases u may be complex!
   zeroindexes=find(u==0);        % added by KS
   u(zeroindexes)=1e-8;           % added by KS 30.4.99  
   u(best)=1;
   c = (c.*u1 - c(Jk)*r(k,:))./u;
   [value,Jk] = max(abs(c));
   Jk=Jk(1);        % only use first element
   best=[best;Jk];
   bestC=[bestC;c(Jk)];
end

r(S,best)=0;      
r(S,Jk) = u(Jk);  
w=zeros(K,1);
if ConditionLimit
   temp=cond(r(:,best));
   if temp>ConditionLimit         % this could happen
      disp([Mfile,': large condition number : ',num2str(log10(temp),3)]);
      w=VSmp2(x,S);         
   else
      w(best) = r(:,best)\bestC;   % 7.6.99 AB, find w without use of inv()
   end
else
   w(best) = r(:,best)\bestC;   % 7.6.99 AB, find w without use of inv()
end   

return;
